# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/loggerwindow.ui'
#
# Created: Tue Oct  1 11:53:09 2013
#      by: pyside-uic 0.2.15 running on PySide 1.1.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_LoggerWindow(object):
    def setupUi(self, LoggerWindow):
        LoggerWindow.setObjectName("LoggerWindow")
        LoggerWindow.resize(648, 469)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        LoggerWindow.setWindowIcon(icon)
        self.gridLayout = QtGui.QGridLayout(LoggerWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtGui.QLabel(LoggerWindow)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.leFilterBy = QtGui.QLineEdit(LoggerWindow)
        self.leFilterBy.setObjectName("leFilterBy")
        self.gridLayout.addWidget(self.leFilterBy, 2, 1, 1, 1)
        self.cbCaseInsensitive = QtGui.QCheckBox(LoggerWindow)
        self.cbCaseInsensitive.setChecked(True)
        self.cbCaseInsensitive.setObjectName("cbCaseInsensitive")
        self.gridLayout.addWidget(self.cbCaseInsensitive, 2, 2, 1, 1)
        self.txtLogHistory = QtGui.QTextBrowser(LoggerWindow)
        self.txtLogHistory.setObjectName("txtLogHistory")
        self.gridLayout.addWidget(self.txtLogHistory, 3, 0, 1, 3)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.btnDebug = QtGui.QPushButton(LoggerWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/script-error.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnDebug.setIcon(icon1)
        self.btnDebug.setCheckable(True)
        self.btnDebug.setChecked(True)
        self.btnDebug.setFlat(True)
        self.btnDebug.setObjectName("btnDebug")
        self.horizontalLayout_2.addWidget(self.btnDebug)
        self.btnInfo = QtGui.QPushButton(LoggerWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/dialog-information.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnInfo.setIcon(icon2)
        self.btnInfo.setCheckable(True)
        self.btnInfo.setChecked(True)
        self.btnInfo.setFlat(True)
        self.btnInfo.setObjectName("btnInfo")
        self.horizontalLayout_2.addWidget(self.btnInfo)
        self.btnWarning = QtGui.QPushButton(LoggerWindow)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/dialog-warning.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnWarning.setIcon(icon3)
        self.btnWarning.setCheckable(True)
        self.btnWarning.setChecked(True)
        self.btnWarning.setFlat(True)
        self.btnWarning.setObjectName("btnWarning")
        self.horizontalLayout_2.addWidget(self.btnWarning)
        self.btnError = QtGui.QPushButton(LoggerWindow)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/dialog-error.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnError.setIcon(icon4)
        self.btnError.setCheckable(True)
        self.btnError.setChecked(True)
        self.btnError.setFlat(True)
        self.btnError.setObjectName("btnError")
        self.horizontalLayout_2.addWidget(self.btnError)
        self.btnCritical = QtGui.QPushButton(LoggerWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/edit-bomb.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnCritical.setIcon(icon5)
        self.btnCritical.setCheckable(True)
        self.btnCritical.setChecked(True)
        self.btnCritical.setFlat(True)
        self.btnCritical.setObjectName("btnCritical")
        self.horizontalLayout_2.addWidget(self.btnCritical)
        self.btnSave = QtGui.QPushButton(LoggerWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/oxygen-icons/document-save-as.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnSave.setIcon(icon6)
        self.btnSave.setObjectName("btnSave")
        self.horizontalLayout_2.addWidget(self.btnSave)
        self.gridLayout.addLayout(self.horizontalLayout_2, 0, 0, 1, 3)

        self.retranslateUi(LoggerWindow)
        QtCore.QMetaObject.connectSlotsByName(LoggerWindow)
        LoggerWindow.setTabOrder(self.btnDebug, self.btnInfo)
        LoggerWindow.setTabOrder(self.btnInfo, self.btnWarning)
        LoggerWindow.setTabOrder(self.btnWarning, self.btnError)
        LoggerWindow.setTabOrder(self.btnError, self.btnCritical)
        LoggerWindow.setTabOrder(self.btnCritical, self.btnSave)
        LoggerWindow.setTabOrder(self.btnSave, self.txtLogHistory)

    def retranslateUi(self, LoggerWindow):
        LoggerWindow.setWindowTitle(QtGui.QApplication.translate("LoggerWindow", "Logs", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("LoggerWindow", "Filter by:", None, QtGui.QApplication.UnicodeUTF8))
        self.cbCaseInsensitive.setText(QtGui.QApplication.translate("LoggerWindow", "Case Insensitive", None, QtGui.QApplication.UnicodeUTF8))
        self.btnDebug.setText(QtGui.QApplication.translate("LoggerWindow", "Debug", None, QtGui.QApplication.UnicodeUTF8))
        self.btnInfo.setText(QtGui.QApplication.translate("LoggerWindow", "Info", None, QtGui.QApplication.UnicodeUTF8))
        self.btnWarning.setText(QtGui.QApplication.translate("LoggerWindow", "Warning", None, QtGui.QApplication.UnicodeUTF8))
        self.btnError.setText(QtGui.QApplication.translate("LoggerWindow", "Error", None, QtGui.QApplication.UnicodeUTF8))
        self.btnCritical.setText(QtGui.QApplication.translate("LoggerWindow", "Critical", None, QtGui.QApplication.UnicodeUTF8))
        self.btnSave.setText(QtGui.QApplication.translate("LoggerWindow", "Save to file", None, QtGui.QApplication.UnicodeUTF8))

import loggerwindow_rc
import mainwindow_rc
