# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/statuspanel.ui'
#
# Created: Mon Oct 14 03:29:37 2013
#      by: pyside-uic 0.2.15 running on PySide 1.1.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_StatusPanel(object):
    def setupUi(self, StatusPanel):
        StatusPanel.setObjectName("StatusPanel")
        StatusPanel.resize(470, 477)
        self.verticalLayout = QtGui.QVBoxLayout(StatusPanel)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblProvider = QtGui.QLabel(StatusPanel)
        self.lblProvider.setStyleSheet("font: bold;")
        self.lblProvider.setWordWrap(True)
        self.lblProvider.setObjectName("lblProvider")
        self.verticalLayout.addWidget(self.lblProvider)
        self.status_rows = QtGui.QWidget(StatusPanel)
        self.status_rows.setEnabled(True)
        self.status_rows.setStyleSheet("")
        self.status_rows.setObjectName("status_rows")
        self.gridLayout = QtGui.QGridLayout(self.status_rows)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_3 = QtGui.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.lblUnread = QtGui.QLabel(self.status_rows)
        self.lblUnread.setObjectName("lblUnread")
        self.gridLayout_3.addWidget(self.lblUnread, 1, 0, 1, 3)
        self.lblMailStatus = QtGui.QLabel(self.status_rows)
        self.lblMailStatus.setStyleSheet("font: bold;")
        self.lblMailStatus.setObjectName("lblMailStatus")
        self.gridLayout_3.addWidget(self.lblMailStatus, 0, 1, 1, 1)
        self.label_4 = QtGui.QLabel(self.status_rows)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 0, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 0, 2, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 1, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem1, 2, 0, 1, 3)
        self.gridLayout.addLayout(self.gridLayout_3, 5, 1, 1, 1)
        self.eip_bandwidth = QtGui.QHBoxLayout()
        self.eip_bandwidth.setSpacing(4)
        self.eip_bandwidth.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.eip_bandwidth.setObjectName("eip_bandwidth")
        self.label_5 = QtGui.QLabel(self.status_rows)
        self.label_5.setText("")
        self.label_5.setPixmap(QtGui.QPixmap(":/images/light/16/down-arrow.png"))
        self.label_5.setObjectName("label_5")
        self.eip_bandwidth.addWidget(self.label_5)
        self.btnDownload = QtGui.QPushButton(self.status_rows)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDownload.sizePolicy().hasHeightForWidth())
        self.btnDownload.setSizePolicy(sizePolicy)
        self.btnDownload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnDownload.setMaximumSize(QtCore.QSize(120, 16777215))
        self.btnDownload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnDownload.setFlat(True)
        self.btnDownload.setObjectName("btnDownload")
        self.eip_bandwidth.addWidget(self.btnDownload)
        spacerItem2 = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth.addItem(spacerItem2)
        self.label_7 = QtGui.QLabel(self.status_rows)
        self.label_7.setText("")
        self.label_7.setPixmap(QtGui.QPixmap(":/images/light/16/up-arrow.png"))
        self.label_7.setObjectName("label_7")
        self.eip_bandwidth.addWidget(self.label_7)
        self.btnUpload = QtGui.QPushButton(self.status_rows)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnUpload.sizePolicy().hasHeightForWidth())
        self.btnUpload.setSizePolicy(sizePolicy)
        self.btnUpload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnUpload.setMaximumSize(QtCore.QSize(120, 16777215))
        self.btnUpload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnUpload.setFlat(True)
        self.btnUpload.setObjectName("btnUpload")
        self.eip_bandwidth.addWidget(self.btnUpload)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth.addItem(spacerItem3)
        self.gridLayout.addLayout(self.eip_bandwidth, 1, 1, 1, 1)
        spacerItem4 = QtGui.QSpacerItem(0, 11, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        self.gridLayout.addItem(spacerItem4, 2, 0, 1, 1)
        self.lblVPNStatusIcon = QtGui.QLabel(self.status_rows)
        self.lblVPNStatusIcon.setMaximumSize(QtCore.QSize(64, 64))
        self.lblVPNStatusIcon.setText("")
        self.lblVPNStatusIcon.setPixmap(QtGui.QPixmap(":/images/light/64/network-eip-down.png"))
        self.lblVPNStatusIcon.setAlignment(QtCore.Qt.AlignCenter)
        self.lblVPNStatusIcon.setObjectName("lblVPNStatusIcon")
        self.gridLayout.addWidget(self.lblVPNStatusIcon, 0, 0, 2, 1)
        self.lblMailIcon = QtGui.QLabel(self.status_rows)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblMailIcon.sizePolicy().hasHeightForWidth())
        self.lblMailIcon.setSizePolicy(sizePolicy)
        self.lblMailIcon.setMinimumSize(QtCore.QSize(64, 64))
        self.lblMailIcon.setMaximumSize(QtCore.QSize(64, 999))
        self.lblMailIcon.setText("")
        self.lblMailIcon.setPixmap(QtGui.QPixmap(":/images/mail-unlocked.png"))
        self.lblMailIcon.setObjectName("lblMailIcon")
        self.gridLayout.addWidget(self.lblMailIcon, 4, 0, 2, 1)
        self.grpMailStatus = QtGui.QGroupBox(self.status_rows)
        self.grpMailStatus.setTitle("")
        self.grpMailStatus.setObjectName("grpMailStatus")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.grpMailStatus)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblLongMailStatus = QtGui.QLabel(self.grpMailStatus)
        self.lblLongMailStatus.setObjectName("lblLongMailStatus")
        self.verticalLayout_2.addWidget(self.lblLongMailStatus)
        self.gridLayout.addWidget(self.grpMailStatus, 7, 0, 1, 2)
        self.globalStatusBox = QtGui.QGroupBox(self.status_rows)
        self.globalStatusBox.setEnabled(False)
        self.globalStatusBox.setObjectName("globalStatusBox")
        self.gridLayout_2 = QtGui.QGridLayout(self.globalStatusBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lblGlobalStatus = QtGui.QLabel(self.globalStatusBox)
        self.lblGlobalStatus.setEnabled(False)
        self.lblGlobalStatus.setWordWrap(True)
        self.lblGlobalStatus.setObjectName("lblGlobalStatus")
        self.gridLayout_2.addWidget(self.lblGlobalStatus, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.globalStatusBox, 3, 0, 1, 2)
        self.eip_controls = QtGui.QHBoxLayout()
        self.eip_controls.setObjectName("eip_controls")
        self.label = QtGui.QLabel(self.status_rows)
        self.label.setObjectName("label")
        self.eip_controls.addWidget(self.label)
        self.lblEIPStatus = QtGui.QLabel(self.status_rows)
        self.lblEIPStatus.setStyleSheet("font: bold;")
        self.lblEIPStatus.setTextFormat(QtCore.Qt.AutoText)
        self.lblEIPStatus.setAlignment(QtCore.Qt.AlignCenter)
        self.lblEIPStatus.setWordWrap(False)
        self.lblEIPStatus.setObjectName("lblEIPStatus")
        self.eip_controls.addWidget(self.lblEIPStatus)
        spacerItem5 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.eip_controls.addItem(spacerItem5)
        self.btnEipStartStop = QtGui.QPushButton(self.status_rows)
        self.btnEipStartStop.setObjectName("btnEipStartStop")
        self.eip_controls.addWidget(self.btnEipStartStop)
        self.gridLayout.addLayout(self.eip_controls, 0, 1, 1, 1)
        self.verticalLayout.addWidget(self.status_rows)
        spacerItem6 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem6)

        self.retranslateUi(StatusPanel)
        QtCore.QMetaObject.connectSlotsByName(StatusPanel)

    def retranslateUi(self, StatusPanel):
        StatusPanel.setWindowTitle(QtGui.QApplication.translate("StatusPanel", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProvider.setText(QtGui.QApplication.translate("StatusPanel", "user@domain.org", None, QtGui.QApplication.UnicodeUTF8))
        self.lblUnread.setText(QtGui.QApplication.translate("StatusPanel", "0 Unread Emails", None, QtGui.QApplication.UnicodeUTF8))
        self.lblMailStatus.setText(QtGui.QApplication.translate("StatusPanel", "Disabled", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("StatusPanel", "Encrypted Mail:", None, QtGui.QApplication.UnicodeUTF8))
        self.btnDownload.setText(QtGui.QApplication.translate("StatusPanel", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))
        self.btnUpload.setText(QtGui.QApplication.translate("StatusPanel", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))
        self.lblLongMailStatus.setText(QtGui.QApplication.translate("StatusPanel", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.lblGlobalStatus.setText(QtGui.QApplication.translate("StatusPanel", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("StatusPanel", "Encrypted Internet: ", None, QtGui.QApplication.UnicodeUTF8))
        self.lblEIPStatus.setText(QtGui.QApplication.translate("StatusPanel", "Off", None, QtGui.QApplication.UnicodeUTF8))
        self.btnEipStartStop.setText(QtGui.QApplication.translate("StatusPanel", "Turn On", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc
