# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/eip_status.ui'
#
# Created: Wed Oct 30 17:39:24 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_EIPStatus(object):
    def setupUi(self, EIPStatus):
        EIPStatus.setObjectName("EIPStatus")
        EIPStatus.resize(470, 157)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(EIPStatus.sizePolicy().hasHeightForWidth())
        EIPStatus.setSizePolicy(sizePolicy)
        self.verticalLayout = QtGui.QVBoxLayout(EIPStatus)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 24)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.btnEipStartStop = QtGui.QPushButton(EIPStatus)
        self.btnEipStartStop.setObjectName("btnEipStartStop")
        self.gridLayout.addWidget(self.btnEipStartStop, 0, 2, 1, 1)
        self.label = QtGui.QLabel(EIPStatus)
        self.label.setMaximumSize(QtCore.QSize(32, 32))
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/images/black/32/earth.png"))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lblEIPStatus = QtGui.QLabel(EIPStatus)
        self.lblEIPStatus.setMaximumSize(QtCore.QSize(16777215, 32))
        self.lblEIPStatus.setStyleSheet("color: rgb(80, 80, 80);")
        self.lblEIPStatus.setWordWrap(True)
        self.lblEIPStatus.setObjectName("lblEIPStatus")
        self.gridLayout.addWidget(self.lblEIPStatus, 3, 1, 1, 1)
        self.lblEIPMessage = QtGui.QLabel(EIPStatus)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblEIPMessage.sizePolicy().hasHeightForWidth())
        self.lblEIPMessage.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setWeight(75)
        font.setBold(True)
        self.lblEIPMessage.setFont(font)
        self.lblEIPMessage.setWordWrap(True)
        self.lblEIPMessage.setObjectName("lblEIPMessage")
        self.gridLayout.addWidget(self.lblEIPMessage, 0, 1, 1, 1)
        self.lblVPNStatusIcon = QtGui.QLabel(EIPStatus)
        self.lblVPNStatusIcon.setMaximumSize(QtCore.QSize(24, 24))
        self.lblVPNStatusIcon.setText("")
        self.lblVPNStatusIcon.setPixmap(QtGui.QPixmap(":/images/black/32/off.png"))
        self.lblVPNStatusIcon.setScaledContents(True)
        self.lblVPNStatusIcon.setObjectName("lblVPNStatusIcon")
        self.gridLayout.addWidget(self.lblVPNStatusIcon, 0, 3, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 1, 1, 1)
        self.eip_bandwidth = QtGui.QWidget(EIPStatus)
        self.eip_bandwidth.setMaximumSize(QtCore.QSize(16777215, 32))
        self.eip_bandwidth.setObjectName("eip_bandwidth")
        self.horizontalLayout = QtGui.QHBoxLayout(self.eip_bandwidth)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.eip_bandwidth_layout = QtGui.QHBoxLayout()
        self.eip_bandwidth_layout.setSpacing(4)
        self.eip_bandwidth_layout.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.eip_bandwidth_layout.setObjectName("eip_bandwidth_layout")
        self.label_5 = QtGui.QLabel(self.eip_bandwidth)
        self.label_5.setText("")
        self.label_5.setPixmap(QtGui.QPixmap(":/images/light/16/down-arrow.png"))
        self.label_5.setObjectName("label_5")
        self.eip_bandwidth_layout.addWidget(self.label_5)
        self.btnDownload = QtGui.QPushButton(self.eip_bandwidth)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDownload.sizePolicy().hasHeightForWidth())
        self.btnDownload.setSizePolicy(sizePolicy)
        self.btnDownload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnDownload.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.btnDownload.setFont(font)
        self.btnDownload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnDownload.setStyleSheet("text-align: left;")
        self.btnDownload.setFlat(True)
        self.btnDownload.setObjectName("btnDownload")
        self.eip_bandwidth_layout.addWidget(self.btnDownload)
        spacerItem1 = QtGui.QSpacerItem(10, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth_layout.addItem(spacerItem1)
        self.label_7 = QtGui.QLabel(self.eip_bandwidth)
        self.label_7.setText("")
        self.label_7.setPixmap(QtGui.QPixmap(":/images/light/16/up-arrow.png"))
        self.label_7.setObjectName("label_7")
        self.eip_bandwidth_layout.addWidget(self.label_7)
        self.btnUpload = QtGui.QPushButton(self.eip_bandwidth)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnUpload.sizePolicy().hasHeightForWidth())
        self.btnUpload.setSizePolicy(sizePolicy)
        self.btnUpload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnUpload.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.btnUpload.setFont(font)
        self.btnUpload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnUpload.setStyleSheet("text-align: left;")
        self.btnUpload.setFlat(True)
        self.btnUpload.setObjectName("btnUpload")
        self.eip_bandwidth_layout.addWidget(self.btnUpload)
        spacerItem2 = QtGui.QSpacerItem(0, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth_layout.addItem(spacerItem2)
        self.horizontalLayout.addLayout(self.eip_bandwidth_layout)
        self.gridLayout.addWidget(self.eip_bandwidth, 2, 1, 1, 3)
        self.verticalLayout.addLayout(self.gridLayout)

        self.retranslateUi(EIPStatus)
        QtCore.QMetaObject.connectSlotsByName(EIPStatus)

    def retranslateUi(self, EIPStatus):
        EIPStatus.setWindowTitle(QtGui.QApplication.translate("EIPStatus", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.btnEipStartStop.setText(QtGui.QApplication.translate("EIPStatus", "Turn On", None, QtGui.QApplication.UnicodeUTF8))
        self.lblEIPStatus.setText(QtGui.QApplication.translate("EIPStatus", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.lblEIPMessage.setText(QtGui.QApplication.translate("EIPStatus", "Traffic is being routed in the clear", None, QtGui.QApplication.UnicodeUTF8))
        self.btnDownload.setText(QtGui.QApplication.translate("EIPStatus", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))
        self.btnUpload.setText(QtGui.QApplication.translate("EIPStatus", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))

import mainwindow_rc
import icons_rc
