# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/mainwindow.ui'
#
# Created: Wed Oct 30 17:39:23 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(524, 722)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(524, 0))
        MainWindow.setMaximumSize(QtCore.QSize(524, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setInputMethodHints(QtCore.Qt.ImhHiddenText)
        MainWindow.setIconSize(QtCore.QSize(128, 128))
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtGui.QScrollArea(self.centralwidget)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QtGui.QFrame.Plain)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 524, 636))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtGui.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.eipWidget = QtGui.QWidget(self.scrollAreaWidgetContents)
        self.eipWidget.setObjectName("eipWidget")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.eipWidget)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame_2 = QtGui.QFrame(self.eipWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setStyleSheet("QFrame{background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 rgba(160, 160, 160, 128), stop:1 rgba(255, 255, 255, 0));}")
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.frame_2)
        self.horizontalLayout_3.setContentsMargins(24, -1, 24, -1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtGui.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setWeight(75)
        font.setBold(True)
        self.label_2.setFont(font)
        self.label_2.setStyleSheet("background-color: rgba(255, 255, 255, 0);")
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.btnEIPPreferences = QtGui.QPushButton(self.frame_2)
        self.btnEIPPreferences.setMaximumSize(QtCore.QSize(48, 20))
        self.btnEIPPreferences.setStyleSheet("")
        self.btnEIPPreferences.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/black/32/gear.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnEIPPreferences.setIcon(icon1)
        self.btnEIPPreferences.setAutoDefault(False)
        self.btnEIPPreferences.setDefault(False)
        self.btnEIPPreferences.setFlat(False)
        self.btnEIPPreferences.setObjectName("btnEIPPreferences")
        self.horizontalLayout_3.addWidget(self.btnEIPPreferences)
        self.verticalLayout_2.addWidget(self.frame_2)
        self.eipLayout = QtGui.QVBoxLayout()
        self.eipLayout.setContentsMargins(12, 0, 12, 0)
        self.eipLayout.setObjectName("eipLayout")
        self.verticalLayout_2.addLayout(self.eipLayout)
        self.verticalLayout.addWidget(self.eipWidget)
        self.line = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.frame = QtGui.QFrame(self.scrollAreaWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setAutoFillBackground(False)
        self.frame.setStyleSheet("QFrame{\n"
"background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 rgba(160, 160, 160, 128), stop:1 rgba(255, 255, 255, 0));\n"
"}")
        self.frame.setObjectName("frame")
        self.horizontalLayout = QtGui.QHBoxLayout(self.frame)
        self.horizontalLayout.setContentsMargins(24, -1, 24, -1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lblLoginProvider = QtGui.QLabel(self.frame)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setWeight(75)
        font.setBold(True)
        self.lblLoginProvider.setFont(font)
        self.lblLoginProvider.setStyleSheet("background-color: rgba(255, 255, 255, 0);")
        self.lblLoginProvider.setObjectName("lblLoginProvider")
        self.horizontalLayout.addWidget(self.lblLoginProvider)
        self.btnPreferences = QtGui.QPushButton(self.frame)
        self.btnPreferences.setMaximumSize(QtCore.QSize(48, 20))
        self.btnPreferences.setStyleSheet("")
        self.btnPreferences.setText("")
        self.btnPreferences.setIcon(icon1)
        self.btnPreferences.setObjectName("btnPreferences")
        self.horizontalLayout.addWidget(self.btnPreferences)
        self.verticalLayout.addWidget(self.frame)
        self.loginLayout = QtGui.QVBoxLayout()
        self.loginLayout.setContentsMargins(-1, 12, -1, -1)
        self.loginLayout.setObjectName("loginLayout")
        self.verticalLayout.addLayout(self.loginLayout)
        self.line_2 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.mailWidget = QtGui.QWidget(self.scrollAreaWidgetContents)
        self.mailWidget.setObjectName("mailWidget")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.mailWidget)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.mailLayout = QtGui.QVBoxLayout()
        self.mailLayout.setSpacing(-1)
        self.mailLayout.setContentsMargins(12, 12, 12, 12)
        self.mailLayout.setObjectName("mailLayout")
        self.verticalLayout_3.addLayout(self.mailLayout)
        self.verticalLayout.addWidget(self.mailWidget)
        spacerItem = QtGui.QSpacerItem(20, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 2)
        self.gridLayout_4 = QtGui.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lblNewUpdates = QtGui.QLabel(self.centralwidget)
        self.lblNewUpdates.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblNewUpdates.setObjectName("lblNewUpdates")
        self.gridLayout_4.addWidget(self.lblNewUpdates, 1, 1, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 1, 0, 1, 1)
        spacerItem2 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 0, 1, 1, 1)
        self.btnMore = QtGui.QPushButton(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnMore.sizePolicy().hasHeightForWidth())
        self.btnMore.setSizePolicy(sizePolicy)
        self.btnMore.setFlat(True)
        self.btnMore.setObjectName("btnMore")
        self.gridLayout_4.addWidget(self.btnMore, 1, 2, 1, 1)
        spacerItem3 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem3, 1, 3, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_4, 0, 0, 1, 2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 524, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.action_preferences = QtGui.QAction(MainWindow)
        self.action_preferences.setObjectName("action_preferences")
        self.action_quit = QtGui.QAction(MainWindow)
        self.action_quit.setObjectName("action_quit")
        self.action_about_leap = QtGui.QAction(MainWindow)
        self.action_about_leap.setObjectName("action_about_leap")
        self.action_help = QtGui.QAction(MainWindow)
        self.action_help.setObjectName("action_help")
        self.action_wizard = QtGui.QAction(MainWindow)
        self.action_wizard.setObjectName("action_wizard")
        self.action_show_logs = QtGui.QAction(MainWindow)
        self.action_show_logs.setObjectName("action_show_logs")
        self.action_create_new_account = QtGui.QAction(MainWindow)
        self.action_create_new_account.setObjectName("action_create_new_account")
        self.menuFile.addAction(self.action_create_new_account)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_quit)
        self.menuHelp.addAction(self.action_help)
        self.menuHelp.addAction(self.action_show_logs)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.action_about_leap)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("MainWindow", "Encrypted Internet", None, QtGui.QApplication.UnicodeUTF8))
        self.lblLoginProvider.setText(QtGui.QApplication.translate("MainWindow", "Login", None, QtGui.QApplication.UnicodeUTF8))
        self.lblNewUpdates.setText(QtGui.QApplication.translate("MainWindow", "There are new updates available, please restart.", None, QtGui.QApplication.UnicodeUTF8))
        self.btnMore.setText(QtGui.QApplication.translate("MainWindow", "More...", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "Help", None, QtGui.QApplication.UnicodeUTF8))
        self.action_preferences.setText(QtGui.QApplication.translate("MainWindow", "Preferences...", None, QtGui.QApplication.UnicodeUTF8))
        self.action_quit.setText(QtGui.QApplication.translate("MainWindow", "&Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.action_about_leap.setText(QtGui.QApplication.translate("MainWindow", "About &Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.action_help.setText(QtGui.QApplication.translate("MainWindow", "&Help", None, QtGui.QApplication.UnicodeUTF8))
        self.action_wizard.setText(QtGui.QApplication.translate("MainWindow", "&Wizard", None, QtGui.QApplication.UnicodeUTF8))
        self.action_show_logs.setText(QtGui.QApplication.translate("MainWindow", "Show &Log", None, QtGui.QApplication.UnicodeUTF8))
        self.action_create_new_account.setText(QtGui.QApplication.translate("MainWindow", "Create a new account...", None, QtGui.QApplication.UnicodeUTF8))

import mainwindow_rc
import locale_rc
