# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/wizard.ui'
#
# Created: Wed Oct 30 17:39:23 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Wizard(object):
    def setupUi(self, Wizard):
        Wizard.setObjectName("Wizard")
        Wizard.resize(536, 510)
        Wizard.setMinimumSize(QtCore.QSize(536, 510))
        Wizard.setMaximumSize(QtCore.QSize(536, 510))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Wizard.setWindowIcon(icon)
        Wizard.setModal(True)
        Wizard.setWizardStyle(QtGui.QWizard.ModernStyle)
        Wizard.setOptions(QtGui.QWizard.IndependentPages)
        self.introduction_page = QtGui.QWizardPage()
        self.introduction_page.setObjectName("introduction_page")
        self.gridLayout = QtGui.QGridLayout(self.introduction_page)
        self.gridLayout.setObjectName("gridLayout")
        self.rdoLogin = QtGui.QRadioButton(self.introduction_page)
        self.rdoLogin.setObjectName("rdoLogin")
        self.gridLayout.addWidget(self.rdoLogin, 3, 0, 1, 1)
        self.label_3 = QtGui.QLabel(self.introduction_page)
        self.label_3.setTextFormat(QtCore.Qt.RichText)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.rdoRegister = QtGui.QRadioButton(self.introduction_page)
        self.rdoRegister.setChecked(True)
        self.rdoRegister.setObjectName("rdoRegister")
        self.gridLayout.addWidget(self.rdoRegister, 2, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 4, 0, 1, 1)
        Wizard.addPage(self.introduction_page)
        self.select_provider_page = WizardPage()
        self.select_provider_page.setObjectName("select_provider_page")
        self.gridLayout_2 = QtGui.QGridLayout(self.select_provider_page)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 0, 1, 1, 1)
        spacerItem3 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem3, 5, 1, 1, 1)
        self.grpCheckProvider = QtGui.QGroupBox(self.select_provider_page)
        self.grpCheckProvider.setObjectName("grpCheckProvider")
        self.gridLayout_3 = QtGui.QGridLayout(self.grpCheckProvider)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_5 = QtGui.QLabel(self.grpCheckProvider)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 3, 0, 1, 1)
        self.label_4 = QtGui.QLabel(self.grpCheckProvider)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 2, 0, 1, 1)
        self.lblProviderInfo = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblProviderInfo.sizePolicy().hasHeightForWidth())
        self.lblProviderInfo.setSizePolicy(sizePolicy)
        self.lblProviderInfo.setMinimumSize(QtCore.QSize(24, 24))
        self.lblProviderInfo.setText("")
        self.lblProviderInfo.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblProviderInfo.setObjectName("lblProviderInfo")
        self.gridLayout_3.addWidget(self.lblProviderInfo, 3, 1, 1, 1)
        self.lblHTTPS = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblHTTPS.sizePolicy().hasHeightForWidth())
        self.lblHTTPS.setSizePolicy(sizePolicy)
        self.lblHTTPS.setMinimumSize(QtCore.QSize(24, 24))
        self.lblHTTPS.setText("")
        self.lblHTTPS.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblHTTPS.setObjectName("lblHTTPS")
        self.gridLayout_3.addWidget(self.lblHTTPS, 2, 1, 1, 1)
        self.lblNameResolution = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblNameResolution.sizePolicy().hasHeightForWidth())
        self.lblNameResolution.setSizePolicy(sizePolicy)
        self.lblNameResolution.setMinimumSize(QtCore.QSize(24, 24))
        self.lblNameResolution.setText("")
        self.lblNameResolution.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblNameResolution.setObjectName("lblNameResolution")
        self.gridLayout_3.addWidget(self.lblNameResolution, 1, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.grpCheckProvider)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 1, 0, 1, 1)
        spacerItem4 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem4, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.grpCheckProvider, 6, 0, 1, 3)
        self.lblProviderSelectStatus = QtGui.QLabel(self.select_provider_page)
        self.lblProviderSelectStatus.setText("")
        self.lblProviderSelectStatus.setObjectName("lblProviderSelectStatus")
        self.gridLayout_2.addWidget(self.lblProviderSelectStatus, 4, 1, 1, 2)
        self.groupBox = QtGui.QGroupBox(self.select_provider_page)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_5 = QtGui.QGridLayout(self.groupBox)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.rbNewProvider = QtGui.QRadioButton(self.groupBox)
        self.rbNewProvider.setChecked(True)
        self.rbNewProvider.setObjectName("rbNewProvider")
        self.gridLayout_5.addWidget(self.rbNewProvider, 0, 0, 1, 1)
        self.rbExistingProvider = QtGui.QRadioButton(self.groupBox)
        self.rbExistingProvider.setObjectName("rbExistingProvider")
        self.gridLayout_5.addWidget(self.rbExistingProvider, 2, 0, 1, 1)
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout_5.addWidget(self.label, 1, 0, 1, 1)
        self.lnProvider = QtGui.QLineEdit(self.groupBox)
        self.lnProvider.setObjectName("lnProvider")
        self.gridLayout_5.addWidget(self.lnProvider, 1, 1, 1, 1)
        self.btnCheck = QtGui.QPushButton(self.groupBox)
        self.btnCheck.setObjectName("btnCheck")
        self.gridLayout_5.addWidget(self.btnCheck, 1, 2, 1, 1)
        self.label_8 = QtGui.QLabel(self.groupBox)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_5.addWidget(self.label_8, 3, 0, 1, 1)
        self.cbProviders = QtGui.QComboBox(self.groupBox)
        self.cbProviders.setEnabled(False)
        self.cbProviders.setObjectName("cbProviders")
        self.gridLayout_5.addWidget(self.cbProviders, 3, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox, 1, 0, 1, 3)
        Wizard.addPage(self.select_provider_page)
        self.provider_info_page = QtGui.QWizardPage()
        self.provider_info_page.setObjectName("provider_info_page")
        self.gridLayout_4 = QtGui.QGridLayout(self.provider_info_page)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lblProviderName = QtGui.QLabel(self.provider_info_page)
        self.lblProviderName.setObjectName("lblProviderName")
        self.gridLayout_4.addWidget(self.lblProviderName, 1, 0, 1, 2)
        spacerItem5 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem5, 6, 1, 1, 1)
        self.lblProviderDesc = QtGui.QLabel(self.provider_info_page)
        self.lblProviderDesc.setMinimumSize(QtCore.QSize(0, 0))
        self.lblProviderDesc.setBaseSize(QtCore.QSize(200, 0))
        self.lblProviderDesc.setWordWrap(True)
        self.lblProviderDesc.setObjectName("lblProviderDesc")
        self.gridLayout_4.addWidget(self.lblProviderDesc, 3, 1, 1, 2)
        self.lblServ = QtGui.QLabel(self.provider_info_page)
        self.lblServ.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblServ.setObjectName("lblServ")
        self.gridLayout_4.addWidget(self.lblServ, 4, 0, 1, 1)
        self.lblServicesOffered = QtGui.QLabel(self.provider_info_page)
        self.lblServicesOffered.setObjectName("lblServicesOffered")
        self.gridLayout_4.addWidget(self.lblServicesOffered, 4, 1, 1, 1)
        spacerItem6 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem6, 4, 2, 1, 1)
        self.label_12 = QtGui.QLabel(self.provider_info_page)
        self.label_12.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 5, 0, 1, 1)
        self.lblProviderPolicy = QtGui.QLabel(self.provider_info_page)
        self.lblProviderPolicy.setObjectName("lblProviderPolicy")
        self.gridLayout_4.addWidget(self.lblProviderPolicy, 5, 1, 1, 1)
        spacerItem7 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem7, 0, 1, 1, 1)
        self.label_6 = QtGui.QLabel(self.provider_info_page)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_4.addWidget(self.label_6, 2, 0, 1, 1)
        self.lblProviderURL = QtGui.QLabel(self.provider_info_page)
        self.lblProviderURL.setObjectName("lblProviderURL")
        self.gridLayout_4.addWidget(self.lblProviderURL, 2, 1, 1, 2)
        self.label_7 = QtGui.QLabel(self.provider_info_page)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_7.setObjectName("label_7")
        self.gridLayout_4.addWidget(self.label_7, 3, 0, 1, 1)
        Wizard.addPage(self.provider_info_page)
        self.setup_provider_page = WizardPage()
        self.setup_provider_page.setObjectName("setup_provider_page")
        self.verticalLayout = QtGui.QVBoxLayout(self.setup_provider_page)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem8 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem8)
        self.lblSetupProviderExpl = QtGui.QLabel(self.setup_provider_page)
        self.lblSetupProviderExpl.setWordWrap(True)
        self.lblSetupProviderExpl.setObjectName("lblSetupProviderExpl")
        self.verticalLayout.addWidget(self.lblSetupProviderExpl)
        spacerItem9 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        self.groupBox_2 = QtGui.QGroupBox(self.setup_provider_page)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_6 = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.lblCheckCaFpr = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblCheckCaFpr.sizePolicy().hasHeightForWidth())
        self.lblCheckCaFpr.setSizePolicy(sizePolicy)
        self.lblCheckCaFpr.setMinimumSize(QtCore.QSize(24, 24))
        self.lblCheckCaFpr.setText("")
        self.lblCheckCaFpr.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblCheckCaFpr.setObjectName("lblCheckCaFpr")
        self.gridLayout_6.addWidget(self.lblCheckCaFpr, 2, 1, 1, 1)
        self.lblDownloadCaCert = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblDownloadCaCert.sizePolicy().hasHeightForWidth())
        self.lblDownloadCaCert.setSizePolicy(sizePolicy)
        self.lblDownloadCaCert.setMinimumSize(QtCore.QSize(24, 24))
        self.lblDownloadCaCert.setText("")
        self.lblDownloadCaCert.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblDownloadCaCert.setObjectName("lblDownloadCaCert")
        self.gridLayout_6.addWidget(self.lblDownloadCaCert, 1, 1, 1, 1)
        self.label_9 = QtGui.QLabel(self.groupBox_2)
        self.label_9.setObjectName("label_9")
        self.gridLayout_6.addWidget(self.label_9, 1, 0, 1, 1)
        self.label_10 = QtGui.QLabel(self.groupBox_2)
        self.label_10.setObjectName("label_10")
        self.gridLayout_6.addWidget(self.label_10, 2, 0, 1, 1)
        self.label_11 = QtGui.QLabel(self.groupBox_2)
        self.label_11.setObjectName("label_11")
        self.gridLayout_6.addWidget(self.label_11, 3, 0, 1, 1)
        self.lblCheckApiCert = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblCheckApiCert.sizePolicy().hasHeightForWidth())
        self.lblCheckApiCert.setSizePolicy(sizePolicy)
        self.lblCheckApiCert.setMinimumSize(QtCore.QSize(24, 24))
        self.lblCheckApiCert.setText("")
        self.lblCheckApiCert.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblCheckApiCert.setObjectName("lblCheckApiCert")
        self.gridLayout_6.addWidget(self.lblCheckApiCert, 3, 1, 1, 1)
        spacerItem10 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem10, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem11 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem11)
        Wizard.addPage(self.setup_provider_page)
        self.register_user_page = WizardPage()
        self.register_user_page.setObjectName("register_user_page")
        self.gridLayout_7 = QtGui.QGridLayout(self.register_user_page)
        self.gridLayout_7.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.gridLayout_7.setContentsMargins(4, -1, -1, -1)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_16 = QtGui.QLabel(self.register_user_page)
        self.label_16.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout_7.addWidget(self.label_16, 3, 0, 1, 1)
        self.lblPassword = QtGui.QLineEdit(self.register_user_page)
        self.lblPassword.setEchoMode(QtGui.QLineEdit.Password)
        self.lblPassword.setObjectName("lblPassword")
        self.gridLayout_7.addWidget(self.lblPassword, 3, 1, 1, 2)
        self.lblPassword2 = QtGui.QLineEdit(self.register_user_page)
        self.lblPassword2.setEchoMode(QtGui.QLineEdit.Password)
        self.lblPassword2.setObjectName("lblPassword2")
        self.gridLayout_7.addWidget(self.lblPassword2, 4, 1, 1, 2)
        self.lblUser = QtGui.QLineEdit(self.register_user_page)
        self.lblUser.setObjectName("lblUser")
        self.gridLayout_7.addWidget(self.lblUser, 2, 1, 1, 2)
        self.label_17 = QtGui.QLabel(self.register_user_page)
        self.label_17.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout_7.addWidget(self.label_17, 4, 0, 1, 1)
        self.btnRegister = QtGui.QPushButton(self.register_user_page)
        self.btnRegister.setObjectName("btnRegister")
        self.gridLayout_7.addWidget(self.btnRegister, 6, 1, 1, 1)
        spacerItem12 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem12, 0, 1, 1, 1)
        spacerItem13 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem13, 7, 1, 1, 1)
        self.label_15 = QtGui.QLabel(self.register_user_page)
        self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout_7.addWidget(self.label_15, 2, 0, 1, 1)
        self.chkRemember = QtGui.QCheckBox(self.register_user_page)
        self.chkRemember.setEnabled(False)
        self.chkRemember.setObjectName("chkRemember")
        self.gridLayout_7.addWidget(self.chkRemember, 5, 1, 1, 2)
        self.lblRegisterStatus = QtGui.QLabel(self.register_user_page)
        self.lblRegisterStatus.setText("")
        self.lblRegisterStatus.setTextFormat(QtCore.Qt.AutoText)
        self.lblRegisterStatus.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lblRegisterStatus.setWordWrap(True)
        self.lblRegisterStatus.setObjectName("lblRegisterStatus")
        self.gridLayout_7.addWidget(self.lblRegisterStatus, 1, 1, 1, 2)
        Wizard.addPage(self.register_user_page)
        self.service_selection = QtGui.QWizardPage()
        self.service_selection.setObjectName("service_selection")
        self.gridLayout_8 = QtGui.QGridLayout(self.service_selection)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.grpServices = QtGui.QGroupBox(self.service_selection)
        self.grpServices.setTitle("Services by PROVIDER")
        self.grpServices.setObjectName("grpServices")
        self.gridLayout_9 = QtGui.QGridLayout(self.grpServices)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.serviceListLayout = QtGui.QVBoxLayout()
        self.serviceListLayout.setObjectName("serviceListLayout")
        self.gridLayout_9.addLayout(self.serviceListLayout, 0, 0, 1, 1)
        self.gridLayout_8.addWidget(self.grpServices, 0, 0, 1, 1)
        Wizard.addPage(self.service_selection)

        self.retranslateUi(Wizard)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.cbProviders.setFocus)
        QtCore.QObject.connect(self.rbNewProvider, QtCore.SIGNAL("toggled(bool)"), self.lnProvider.setFocus)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.lnProvider.setDisabled)
        QtCore.QObject.connect(self.rbNewProvider, QtCore.SIGNAL("toggled(bool)"), self.cbProviders.setDisabled)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.btnCheck.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(Wizard)
        Wizard.setTabOrder(self.lblUser, self.lblPassword)
        Wizard.setTabOrder(self.lblPassword, self.lblPassword2)
        Wizard.setTabOrder(self.lblPassword2, self.btnRegister)
        Wizard.setTabOrder(self.btnRegister, self.rdoRegister)
        Wizard.setTabOrder(self.rdoRegister, self.rdoLogin)

    def retranslateUi(self, Wizard):
        Wizard.setWindowTitle(QtGui.QApplication.translate("Wizard", "Bitmask first run", None, QtGui.QApplication.UnicodeUTF8))
        self.introduction_page.setTitle(QtGui.QApplication.translate("Wizard", "Welcome", None, QtGui.QApplication.UnicodeUTF8))
        self.introduction_page.setSubTitle(QtGui.QApplication.translate("Wizard", "This is the Bitmask first run wizard", None, QtGui.QApplication.UnicodeUTF8))
        self.rdoLogin.setText(QtGui.QApplication.translate("Wizard", "Log In with my credentials", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Wizard", "<html><head/><body><p>Now we will guide you through some configuration that is needed before you can connect for the first time.</p><p>If you ever need to modify these options again, you can find the wizard in the <span style=\" font-style:italic;\">\'Settings\'</span> menu from the main window.</p><p>Do you want to <span style=\" font-weight:600;\">sign up</span> for a new account, or <span style=\" font-weight:600;\">log in</span> with an already existing username?</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.rdoRegister.setText(QtGui.QApplication.translate("Wizard", "Sign up for a new account", None, QtGui.QApplication.UnicodeUTF8))
        self.select_provider_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider selection", None, QtGui.QApplication.UnicodeUTF8))
        self.select_provider_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Please enter the domain of the provider you want to use for your connection", None, QtGui.QApplication.UnicodeUTF8))
        self.grpCheckProvider.setTitle(QtGui.QApplication.translate("Wizard", "Checking for a valid provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("Wizard", "Getting provider information", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("Wizard", "Can we establish a secure connection?", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Wizard", "Can we reach this provider?", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("Wizard", "Configure or select a provider", None, QtGui.QApplication.UnicodeUTF8))
        self.rbNewProvider.setText(QtGui.QApplication.translate("Wizard", "Configure new provider:", None, QtGui.QApplication.UnicodeUTF8))
        self.rbExistingProvider.setText(QtGui.QApplication.translate("Wizard", "Use existing one:", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Wizard", "https://", None, QtGui.QApplication.UnicodeUTF8))
        self.btnCheck.setText(QtGui.QApplication.translate("Wizard", "Check", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("Wizard", "https://", None, QtGui.QApplication.UnicodeUTF8))
        self.provider_info_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider Information", None, QtGui.QApplication.UnicodeUTF8))
        self.provider_info_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Description of services offered by this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderName.setText(QtGui.QApplication.translate("Wizard", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderDesc.setText(QtGui.QApplication.translate("Wizard", "Desc", None, QtGui.QApplication.UnicodeUTF8))
        self.lblServ.setText(QtGui.QApplication.translate("Wizard", "<b>Services offered:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblServicesOffered.setText(QtGui.QApplication.translate("Wizard", "services", None, QtGui.QApplication.UnicodeUTF8))
        self.label_12.setText(QtGui.QApplication.translate("Wizard", "<b>Enrollment policy:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderPolicy.setText(QtGui.QApplication.translate("Wizard", "policy", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("Wizard", "<b>URL:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderURL.setText(QtGui.QApplication.translate("Wizard", "URL", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("Wizard", "<b>Description:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.setup_provider_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider setup", None, QtGui.QApplication.UnicodeUTF8))
        self.setup_provider_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Gathering configuration options for this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.lblSetupProviderExpl.setText(QtGui.QApplication.translate("Wizard", "We are downloading some bits that we need to establish a secure connection with the provider for the first time.", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("Wizard", "Setting up provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("Wizard", "Getting info from the Certificate Authority", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("Wizard", "Do we trust this Certificate Authority?", None, QtGui.QApplication.UnicodeUTF8))
        self.label_11.setText(QtGui.QApplication.translate("Wizard", "Establishing a trust relationship with this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.register_user_page.setTitle(QtGui.QApplication.translate("Wizard", "Register new user", None, QtGui.QApplication.UnicodeUTF8))
        self.register_user_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Register a new user with provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_16.setText(QtGui.QApplication.translate("Wizard", "<b>Password:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_17.setText(QtGui.QApplication.translate("Wizard", "<b>Re-enter password:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.btnRegister.setText(QtGui.QApplication.translate("Wizard", "Register", None, QtGui.QApplication.UnicodeUTF8))
        self.label_15.setText(QtGui.QApplication.translate("Wizard", "<b>Username:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.chkRemember.setText(QtGui.QApplication.translate("Wizard", "Remember my username and password", None, QtGui.QApplication.UnicodeUTF8))
        self.service_selection.setTitle(QtGui.QApplication.translate("Wizard", "Service selection", None, QtGui.QApplication.UnicodeUTF8))
        self.service_selection.setSubTitle(QtGui.QApplication.translate("Wizard", "Please select the services you would like to have", None, QtGui.QApplication.UnicodeUTF8))

from wizardpage import WizardPage
import mainwindow_rc
import locale_rc
